/* Copyright (C) 2005-2006  Emiliano Betti and Marco Cesati
   Copyright (C) 2003-2004  Marco Cesati
   Copyright (C) 2002-2003  Shell Technologies s.r.l.

   Authors:
            Emiliano Betti <emilianobetti@gmail.com>
            Marco Cesati   <marcocesati@gmail.com>

   Driver for the Galil 1800 PCI motion controllers (http://www.galilmc.com/)

   This is a driver for the Linux kernel 2.6. It is based on an older driver
   for the kernel 2.4, which in turn was loosely inspired by Galil's own
   Linux driver, as well as by a bunch of PCI drivers in the kernel source code

   This program is free software; you can redistribute  it and/or modify it
   under  the terms of  the GNU General  Public License as published by the
   Free Software Foundation.
*/

#include "galil1800_core.h"
#include "galil1800_sysfs.h"

#include <linux/pci.h>


/* Device file names */
const char galil1800_dev_name[GALIL1800_MINOR_PER_CARD][6] =
					{ "card", "event", "info" };


/* "Show" method for default class_device_attribute "dev" */
static ssize_t galil1800_show_dev(struct class_device *cd, char *buf)
{
	struct galil1800_devfile_state *df_s =
		(struct galil1800_devfile_state *)(cd->class_data);
	return print_dev_t(buf, df_s->cdev.dev);
}

/* Call at class release */
static void galil1800_sysfs_class_release(struct class *c)
{
	kfree(c->class_dev_attrs);
}

/* Call at class_device release */
static void galil1800_sysfs_release(struct class_device *cd)
{
	return;
}

/* Register a class for this module */
int galil1800_sysfs_class_register(struct class *c)
{
	int ret;

	c->name = GALIL1800_CLASS_NAME;
	c->owner = THIS_MODULE;

	c->class_attrs=0;
	c->class_dev_attrs=kcalloc(2, sizeof(struct class_device_attribute),
								GFP_KERNEL);
	if(!(c->class_dev_attrs)) {
		printk(KERN_ERR PFX "galil1800_sysfs_class_init:"
				" alloc class_dev_attrs failed\n");
		ret = -ENOMEM;
		goto out_dev_attrs;
	}

	c->class_dev_attrs[0].attr.name = "dev";
	c->class_dev_attrs[0].attr.owner = THIS_MODULE;
	c->class_dev_attrs[0].attr.mode = S_IRUGO;
	c->class_dev_attrs[0].show = galil1800_show_dev;
	c->class_dev_attrs[0].store = NULL;

	c->class_release = galil1800_sysfs_class_release;
	c->release = galil1800_sysfs_release;

	ret=class_register(c);
	if(ret) {
		printk(KERN_ERR PFX "cannot register the module"
						" class in sysfs\n");
		goto out_reg;
	}

	return 0;
out_reg:
	kfree(c->class_dev_attrs);
out_dev_attrs:
	return ret;
}

/* Register a class_device:
   @cdev_index must be GALIL1800_CARD_DEVICE or
   GALIL1800_EVENT_DEVICE or GALIL1800_INFO_DEVICE */
int galil1800_sysfs_class_dev_register(struct galil1800_state *s,
							int cdev_index)
{
	int ret;
	struct galil1800_devfile_state *df_s = s->devfile_state + cdev_index;
	struct class_device *cd = &df_s->class_dev;

	cd->class = s->sysfs_class;
	cd->dev = &(s->dev->dev);
	cd->class_data = df_s;
	sprintf(cd->class_id, "%s%d", galil1800_dev_name[cdev_index], s->index);

	if((ret=class_device_register(cd))) {
		printk(KERN_ERR PFX "cannot register the class device"
					" \"%s\" in sysfs\n", cd->class_id);
		return ret;
	}
	return 0;
}
